# LOF-Test programmiert (Beispiel "Blutdruck", Teil 3)
# Funktionsdefinition
LOF <- function(mod) ## LOF eines gegebenen linearen Modells
{ if (!class(mod)=="lm") {
  # Fehlerabfang, falls versehentlich KEIN lineares Modell übergeben wird
  stop("LOF function currently only implemented for lm objects.")
}
  # Extrahiere die Designmatrix des gegebenen Modells
  X <- model.matrix(mod)
  # Kreiere eine Faktorvariable, deren Stufen die 
  # verschiedenen Designpunkte sind
  # 1. Trick: Mehrdimensionale Designpunkte werden durch 
  # einfaches Aneinanderreihen zu einem Textstring, 
  # so wird zB der Designpunkt x=2,y=3,z=4 zu “2.3.4”
  # 2. Trick: Nutzung von apply, welches bei einer Matrix eine gegebene Funktion
  # FUN entweder pro Zeile (MARGIN=1) oder Spalte (MARGIN=2) durchführt
  # In diesem Fall ist die gegebene Funktion "paste"
  factorX <- factor(apply(X,
                          MARGIN=1,
                          FUN=function(x) paste(x,collapse = "."))) 
  if (length(levels(factorX)) == dim(X)[1]) {
    ## Fehlerabfang, falls keine Mehrfachversuche vorhanden sind
    print("LOF test not possible, no replication.")
    return(anova(mod))
  }
  
  anova.original <- anova(mod)
  # Berechnung des pure Error mit der Faktorvariablen factorX
  y <- mod$model[,1] # Extrahiere die Response-Variable aus dem gegebenen Modell
  # Extrahiere die Kennzahlen zum pure Error aus der Residualinformation eines
  # maximal spezifizierten Modelles
  anova.pe <- data.frame(anova(lm(y~ 0 + factorX))["Residuals",])
  rownames(anova.pe) <- "  Pure Error"
  colnames(anova.pe) <- colnames(anova.original)
  # Nutze die additive Zerlegung der Reststreuung bei Mehrfachversuchen, 
  # um SS(LOF) und DF(LOF) zu berechnen.
  anova.lof <- anova(mod)["Residuals",] - anova.pe
  # Berechne die MS(LOF) durch die Formel SS(LOF)/DF(LOF)
  anova.lof["Mean Sq"] <- anova.lof["Sum Sq"] / anova.lof["Df"]
  rownames(anova.lof) <- "  Lack of Fit"
  # Berechne die F Statistik
  anova.lof["F value"] <- (anova.lof["Sum Sq"]/anova.lof["Df"])/
    (anova.pe["Sum Sq"]/anova.pe["Df"])
  # Ermittle den dazugehörigen P-Wert (pf ist die von R bereitgestellte 
  # Verteilungsfunktion der F-Verteilung)
  anova.lof["Pr(>F)"] <- pf(as.numeric(anova.lof["F value"]),
                            df1=as.numeric(anova.lof["Df"]), 
                            df2=as.numeric(anova.pe["Df"]),  
                            lower.tail=FALSE)
  return(rbind(anova.original,anova.lof,anova.pe))
  
}
