/* Stichprobenumfang zur Erreichung einer vorgegebenen Messgenauigkeit fr eine Mittelwertverschiebung, 4 Kurven */
%macro stich(alpha,ergebnis);
filename ergebnis &ergebnis;

/* Daten */
data d1(keep=x kurve n) anno(keep=xi y s p f text xsys ysys style);
length text $ 25;
      alpha=&alpha;
      xsys='5'; ysys='5'; xi=40; y=85; f='move '; output anno;
      s=1.5; f='label'; p='C'; style='swiss'; text='alpha='!!left(put(alpha,4.2)); output anno;
do kurve = 1 to 6;
  select;
    when(kurve in (1,2)) do; beta=0.10; xwindow=40; ywindow=70; p='6'; m=30; end;
    when(kurve in (3,4)) do; beta=0.20; xwindow=40; ywindow=55; p='6'; m=20; end;
    when(kurve in (5,6)) do; beta=0.30; xwindow=20; ywindow=30; p='5'; m=20; end;
    otherwise;
  end;
  select;
    when(kurve in (1,3,5)) exakt=1;
    otherwise exakt=0;
  end;
  do n=3 to 51;
    select(exakt); 
      when(1)   x=(tinv(1-alpha,n-1)+tinv(1-beta,n-1))/n**.5;
      otherwise x=(probit(1-alpha)  +probit(1-beta))/n**.5;
    end;
    output d1;
    if n=m then do;
      xsys='2'; ysys='2'; xi=x; y=n; f='move '; output anno;
      xsys='5'; ysys='5'; xi=xwindow; y=ywindow; f='draw '; s=1; output anno;
      s=1.5; f='label'; style='swiss'; text='1-beta='!!left(put(1-beta,4.2));
        output anno;
    end;
  end; 
end;

/* Grafische Optionen */
goptions device=jpeg gsfname=ergebnis;
goptions colors=(black);
title;
axis1 label=none
      offset=(0cm)
      order=0 to 50 by 10 minor=none
      value=(font=zapf height=0.7cm);
axis2 label=none
      /*label=(font=zapf height=2 "z") */
      order=0.25 to 1.75 by 0.25 minor=none
      value=(font=zapf height=0.7cm);
symbol1 V=NONE I=JOIN width=2 L=1;
symbol2 V=NONE I=JOIN width=2 L=2;
symbol3 V=NONE I=JOIN width=2 L=1;
symbol4 V=NONE I=JOIN width=2 L=2;
symbol5 V=NONE I=JOIN width=2 L=1;
symbol6 V=NONE I=JOIN width=2 L=2;
symbol7 V=NONE I=JOIN width=2 L=1;
symbol8 V=NONE I=JOIN width=2 L=2;


/* Plot */
proc gplot data=d1
  annotate=anno(rename=(f=function p=position s=size xi=x));
  plot n*x=kurve/haxis=axis2 href=.5 to 1.75 by 0.25
                 vaxis=axis1 vref=10 to 50 by 10
                 nolegend;
run;
quit;
%mend stich;
%stich(0.05,"S_53-5.jpg");
%stich(0.01,"S_53-1.jpg");


